@@ -4,10 +4,24 @@ from pyFBS.utility import *
 import os
 import requests
 import shutil
+import h5py    
 from tqdm import tqdm
 
+LAB_FOLDER = "lab_testbench"
+LAB_FILES = {"FEM": ["A.full", "A.rst", "AB.full", "AB.rst", "B.full", "B.rst"],
+                 "STL": ["A.stl", "B.stl", "AB.stl"],
+                 "Measurements": ["AM_Measurements.xlsx","ammeasurements.xlsx", "coupling_example.xlsx", "decoupling_example.xlsx",
+                                  "TPA_synt.xlsx", "Y_A.p", "Y_B.p", "Y_AB.p"]}
 
-def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
+AUTOMOTIVE_FOLDER = "automotive_testbench"
+AUTOMOTIVE_FILES = {"FEM": ["EM.full", "EM.rst", "RM.full", "RM.rst", "TM.full", "TM.rst"],
+                 "STL": ["engine_mount.stl", "receiver.stl", "roll_mount.stl", "shaker_only.stl", "source.stl",
+                         "transmission_mount.stl", "ts.stl"],
+                 "Measurements": ["A.p", "A.xlsx", "AB_ref.p", "AB_ref.xlsx", "BTS.p", "BTS.xlsx", "B_ref.p",
+                                  "ODS.p", "ODS.xlsx", "TS.p", "TS.xlsx", "TS.xlsx", "frame_rubbermounts.p",
+                                  "frame_rubbermounts_sourceplate.p", "modal.xlsx"]}
+
+def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input,chn_input = False):
     """
     Loads an Universal File Format .uff file from PAK system and parses the data in arrays and DataFrames
 
@@ -29,7 +43,14 @@ def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
     uff_file = pyuff.UFF(uff_file_data)
     data = uff_file.read_sets()
 
-    chn_dof = len(data_output["x"]) # * 3  # triax acc
+    chn_dof = len(data_output["x"]) 
+    
+    # assuming tri-axial accelerometers
+    if chn_input:
+        _value = 1
+        chn_dof *= 3
+    else:
+        _value = 3
     imp_dof = len(data_input["x"])
 
     Directions = {0: "None", 1: "+X", 2: "+Y", 3: "+Z", -1: "-X", -2: "-Y", -3: "-Z"}
@@ -49,8 +70,8 @@ def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
     for _out in range(chn_dof):
         for _in in range(imp_dof):
             Node_Number = data[i]['rsp_node']
-            Name = 'S' + str(math.ceil(Node_Number / 3)) + " " + Directions[data[i]['rsp_dir']]
-
+            Name = 'S' + str(math.ceil(Node_Number / _value)) + " " + Directions[data[i]['rsp_dir']]
+            
             out_resp[_out, :] = Directions_array[data[i]['rsp_dir']]
             in_resp[_in, :] = Directions_array[data[i]['ref_dir']]
 
@@ -75,7 +96,6 @@ def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
         for i in range(3):
             out_dir = out_resp[_out * 3 + i]
             out_pos = [data_output["x"][_out], data_output["y"][_out], data_output["z"][_out]]
-
             data_chn = np.asarray([[out_N[_out * 3 + i], None, None, out_N[_out * 3 + i].split(" ")[1], None, None,
                                     None, None, None, out_pos[0],
                                     out_pos[1], out_pos[2], out_dir[0], out_dir[1], out_dir[2]]])
@@ -83,7 +103,15 @@ def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
             df_row = pd.DataFrame(data=data_chn, columns=columns_chann)
             df = df.append(df_row, ignore_index=True)
 
-    df_chn = df
+    if chn_input:
+        df_chn = df
+        df_acc = generate_sensors_from_channels(df_chn)
+    else:
+        df_chn = df
+        df_acc = None
+        
+        
+
 
     # parse impact data
     columns_chann = ["Name", "Description", "Type", "DirectionLabel", "Quantity", "Unit", "Component", "NodeNumber",
@@ -105,7 +133,6 @@ def load_uff_file_PAK(uff_file_data,uff_file_output,uff_file_input):
 
     df_imp = df
 
-    df_acc = generate_sensors_from_channels(df_chn)
 
     return freq,FRF,df_chn,df_imp,df_acc
 
@@ -114,16 +141,16 @@ def download_automotive_testbench(overwrite=False):
     """
 
     """
-    folder_name = "automotive_testbench"
+    folder_name = AUTOMOTIVE_FOLDER #"automotive_testbench"
 
     url_automotive = r'https://gitlab.com/pyFBS/pyFBS_data/-/raw/master/automotive_testbench/'
 
-    url_a_sub = {"FEM": ["EM.full", "EM.rst", "RM.full", "RM.rst", "TM.full", "TM.rst"],
-                 "STL": ["engine_mount.stl", "receiver.stl", "roll_mount.stl", "shaker_only.stl", "source.stl",
-                         "transmission_mount.stl", "ts.stl"],
-                 "Measurements": ["A.p", "A.xlsx", "AB_ref.p", "AB_ref.xlsx", "BTS.p", "BTS.xlsx", "B_ref.p",
-                                  "ODS.p", "ODS.xlsx", "TS.p", "TS.xlsx", "TS.xlsx", "frame_rubbermounts.p",
-                                  "frame_rubbermounts_sourceplate.p", "modal.xlsx"]}
+    url_a_sub = AUTOMOTIVE_FILES #{"FEM": ["EM.full", "EM.rst", "RM.full", "RM.rst", "TM.full", "TM.rst"],
+                # "STL": ["engine_mount.stl", "receiver.stl", "roll_mount.stl", "shaker_only.stl", "source.stl",
+                #         "transmission_mount.stl", "ts.stl"],
+                # "Measurements": ["A.p", "A.xlsx", "AB_ref.p", "AB_ref.xlsx", "BTS.p", "BTS.xlsx", "B_ref.p",
+                #                  "ODS.p", "ODS.xlsx", "TS.p", "TS.xlsx", "TS.xlsx", "frame_rubbermounts.p",
+                #                  "frame_rubbermounts_sourceplate.p", "modal.xlsx"]}
 
     # remove folder if overwrite
     if os.path.isdir(folder_name) and overwrite:
@@ -156,14 +183,14 @@ def download_lab_testbench(overwrite=False):
     Download laboratory testbench files
 
     """
-    folder_name = "lab_testbench"
+    folder_name = LAB_FOLDER #"lab_testbench"
 
     url_lab = r'https://gitlab.com/pyFBS/pyFBS_data/-/raw/master/lab_testbench/'
 
-    url_l_sub = {"FEM": ["A.full", "A.rst", "AB.full", "AB.rst", "B.full", "B.rst"],
-                 "STL": ["A.stl", "B.stl", "AB.stl"],
-                 "Measurements": ["AM_Measurements.xlsx","ammeasurements.xlsx", "coupling_example.xlsx", "decoupling_example.xlsx",
-                                  "TPA_synt.xlsx", "Y_A.p", "Y_B.p", "Y_AB.p"]}
+    url_l_sub = LAB_FILES #{"FEM": ["A.full", "A.rst", "AB.full", "AB.rst", "B.full", "B.rst"],
+                # "STL": ["A.stl", "B.stl", "AB.stl"],
+                # "Measurements": ["AM_Measurements.xlsx","ammeasurements.xlsx", "coupling_example.xlsx", "decoupling_example.xlsx",
+                #                  "TPA_synt.xlsx", "Y_A.p", "Y_B.p", "Y_AB.p"]}
 
     # remove folder if overwrite
     if os.path.isdir(folder_name) and overwrite:
@@ -192,3 +219,75 @@ def download_lab_testbench(overwrite=False):
                 with open(folder_name + os.sep + sub_dir + os.sep + '%s' % filename, 'wb') as fout:
                     fout.write(r._content)
 
+def load_hdf_Math(file_name,group_name = "Math"):
+    """
+    Loads HDF file format - Math (FRFs from Modal test), when exported from Dewesoft software.
+
+    :param uff_file_data: A filename of the .hdf file containing information of FRFs
+    :type uff_file_data: str
+    """
+
+    f = h5py.File(file_name,'r+')
+
+    group = f["Math"]
+    keys = []
+    for key in group.keys():
+        keys.append(key)
+
+    arr_out = []
+    arr_in = []
+
+    for key in keys[::2]:
+        _file = key.split("_")
+        name_real = key
+        name_imag = key + "_2"
+        
+        ch_out = _file[2][:-2]
+        ch_in = _file[3][:-2]
+        
+        arr_out.append(int(ch_out))
+        arr_in.append(int(ch_in))
+        
+    _out = np.max(arr_out) - np.min(arr_out) + 1
+    _in = np.max(arr_in) - np.min(arr_in) + 1
+    _f = len(group[keys[0]][()][0])
+
+    Y = np.zeros((_f,_out,_in),dtype = complex)
+    for key in keys[::2]:
+        
+        _file = key.split("_")
+        name_real = key
+        name_imag = key + "_2"
+        
+        ch_out = _file[2][:-2]
+        ch_in = _file[3][:-2]
+        
+        resp = (group[name_real][()]+group[name_imag][()]*1j).T
+        Y[:,int(ch_out)-1,int(ch_in)-1] = resp[:,0]
+    
+    return Y, keys 
+    
+def load_hdf_AI(file_name,group_name = "AI"):
+    """
+    Loads HDF file format - Analog Input, when exported from Dewesoft software.
+
+    :param uff_file_data: A filename of the .hdf file containing information of FRFs
+    :type uff_file_data: str
+    """
+    
+    f = h5py.File(file_name,'r+')
+
+    group = f[group_name]
+
+    keys = []
+    for key in group.keys():
+        keys.append(key)
+    
+    acc = []
+    for i,chn in enumerate(group.keys()): 
+        group[chn][()]
+        acc.append(group[chn][()])
+        
+    acc = np.asarray(acc)
+    
+    return acc, keys
\ No newline at end of file
